package pwc.taxtech.atms.vat.service.impl;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.datasource.ShardingContextHolder;
import pwc.taxtech.atms.common.util.SnowFlake;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.*;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.entitiy.*;
import pwc.taxtech.atms.exception.NotSupportedException;
import pwc.taxtech.atms.service.impl.CellConfigTranslater;
import pwc.taxtech.atms.vat.dao.CellDataSourceMapper;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.ReportService;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaHelper;

import javax.swing.text.html.Option;
import java.math.BigDecimal;
import java.sql.Struct;
import java.text.Normalizer;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class ReportServiceImpl extends VatAbstractService implements ReportService {
    @Override
    public OperationResultDto<List<ReportDto>> getReportTemplate(String projectID, EnumServiceType serviceType, Integer periodParam) {
        int period = periodParam != null ? periodParam : 0;
        OperationResultDto<List<ReportDto>> operationResult = new OperationResultDto<>();
        Project project = projectMapper.selectByPrimaryKey(projectID);
        String organizationID = project == null ? null : project.getOrganizationID();
        if (organizationID == null) {
            operationResult.setResult(false);
            operationResult.setResultMsg("NoOrganization");
            return operationResult;
        }

        Long templateGroupID;

        /*
        对于VAT，从Rule Engine中获取TemplateGroupID
        对于其他Service Type，直接从ProjectServiceType中获取
         */
        if (serviceType.equals(EnumServiceType.VAT)) {
            TaxPayerReportRuleExample example = new TaxPayerReportRuleExample();
            example.createCriteria().andOrganizationIdLike(organizationID);
            List<TaxPayerReportRule> taxPayerReportRules = taxPayerReportRuleMapper.selectByExample(example);
            Optional<TaxPayerReportRule> taxPayerReportRule = taxPayerReportRules.stream().findFirst();
            if (!taxPayerReportRule.isPresent()) {
                Optional<TaxPayerReportRule> defaultRule = taxPayerReportRules.stream().filter(x -> x.getIsDefault()).findFirst();
                if (!defaultRule.isPresent()) {
                    operationResult.setResultMsg("NoDefaultRule");
                    return operationResult;
                }

                templateGroupID = defaultRule.get().getTemplateGroupId();
            } else {
                templateGroupID = taxPayerReportRule.get().getTemplateGroupId();
            }

        } else {
            String serviceTypeStr = serviceType.getCode().toString();
            ProjectServiceTypeExample projectServiceTypeExample = new ProjectServiceTypeExample();
            projectServiceTypeExample.createCriteria().andServiceTypeIDEqualTo(serviceTypeStr).andProjectIDEqualTo(projectID);
            Optional<Long> tempVlaue = projectServiceTypeMapper.selectByExample(projectServiceTypeExample).stream().map(ProjectServiceType::getTemplateGroupID).findFirst();
            if (tempVlaue.isPresent()) {
                templateGroupID = tempVlaue.get();
            } else {
                templateGroupID = 0L;
            }
        }

        String dbName = ShardingContextHolder.getDataSourceKey();
        Map<String, Object> map = new HashMap<>();
        map.put("dbName", dbName);
        map.put("period", period);
        map.put("projectID", projectID);
        map.put("templateGroupID", templateGroupID);

        List<ReportDto> reportDtos = templateMapper.getTemplateLeftJoinReport(map);
        if (reportDtos.isEmpty()) {
            operationResult.setResultMsg("No Template");
            return operationResult;
        }

        operationResult.setResult(true);
        operationResult.setData(reportDtos);
        return operationResult;
    }

    @Override
    public OperationResultDto updateConfig(String projectId, Integer period, Boolean ifDeleteManualDataSource, String generator) {
        OperationResultDto result = new OperationResultDto();

        if (period == null) {
            result.setResultMsg("peirod is null");
            return result;
        }

        Project project = projectMapper.selectByPrimaryKey(projectId);

        if (project == null) {
            result.setResultMsg("NoProject");
            return result;
        }

        List<Long> exceptTemplateIDs = templateMapper.getIDsForExceptTemplate();
        String strExceptTemplateIDs = StringUtils.EMPTY;
        if (exceptTemplateIDs != null && exceptTemplateIDs.size() > 0) {
            strExceptTemplateIDs = StringUtils.join(exceptTemplateIDs, ",");
        } else {
            strExceptTemplateIDs = "''";
        }

        //todo: according to projectId get the templateGroupId
        Long templateGroupID = projectMapper.getTemplateGroupIdByProject(projectId, EnumServiceType.VAT.getCode());
        if (templateGroupID != null && templateGroupID != 0) {
            // 先进行数据清理，包括period开头的所有报表配置表 条件Period
//            dataSourceMapper.clearFormulaBlockWithPeriod(period);
            //todo: reopen it later
            //dataSourceMapper.clearTaxRuleSettingWithPeriod(period);
//            dataSourceMapper.clearCellTemplateWithPeriod(period, strExceptTemplateIDs);
//            dataSourceMapper.clearCellTemplateConfigWithPeriod(period, strExceptTemplateIDs);
//            dataSourceMapper.clearTemplateWithPeriod(period, strExceptTemplateIDs);
            //todo: reopen it later
            //dataSourceMapper.clearTaxPayerReportWithPeriod(period);
//            dataSourceMapper.clearCellDataSourceDataWithPeriod(period);
//            dataSourceMapper.clearDataSourceDetailWithPeriod(period);
//            dataSourceMapper.clearDataSourceWithPeriod(period);
//            dataSourceMapper.clearCellDataWithPeriod(strExceptTemplateIDs, period);
//            dataSourceMapper.clearReportWithPeriod(period);
            dataSourceMapper.clearDataWithPeriod(strExceptTemplateIDs, period);
            // 根据templategroupid 把 template 插入到 periodTemplate
            TemplateExample example = new TemplateExample();
            example.createCriteria().andTemplateGroupIdEqualTo(templateGroupID);
            List<Template> templates = templateMapper.selectByExample(example);
            List<PeriodTemplate> periodTemplateList = new ArrayList<>();
            int i = 0;
            Long startTimeOut = System.currentTimeMillis();
            logger.debug("templates copy start: " + startTimeOut);
            for (Template template : templates) {
                i++;
                Long startTime = System.currentTimeMillis();
                logger.debug("template copy start: " + startTime);
                PeriodTemplate periodTemplate = new PeriodTemplate();
                CommonUtils.copyProperties(template, periodTemplate);
                periodTemplate.setId(distributedIDService.nextId());
                periodTemplate.setPeriod(period);
//                periodTemplate.setName(template.getName());
//                periodTemplate.setCode(template.getCode());
//                periodTemplate.setPath(template.getPath());
//                periodTemplate.setReportType(template.getReportType());
//                periodTemplate.setTemplateGroupId(template.getTemplateGroupId());
//                periodTemplate.setOrderIndex(template.getOrderIndex());
//                periodTemplate.setCreateTime(template.getCreateTime());
//                periodTemplate.setUpdateTime(template.getUpdateTime());
//                periodTemplate.setIsSystemType(template.getIsSystemType());
//                periodTemplate.setIsActiveAssociation(template.getIsActiveAssociation());
//                periodTemplate.setParentId(template.getParentId());
                periodTemplate.setTemplateId(template.getId());
//                periodTemplate.setCreateBy(template.getCreateBy());
//                periodTemplate.setUpdateBy(template.getUpdateBy());
                periodTemplateList.add(periodTemplate);
                logger.debug("template copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
            }
            logger.debug("templates copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

            List<Long> templateIDList = periodTemplateList.stream().map(PeriodTemplate::getTemplateId).collect(Collectors.toList());
            CellTemplateExample cellTemplateExample = new CellTemplateExample();
            cellTemplateExample.createCriteria().andReportTemplateIdIn(templateIDList);
            List<CellTemplate> cellTemplateList = cellTemplateMapper.selectByExample(cellTemplateExample);
            List<PeriodCellTemplate> periodCellTemplateList = new ArrayList<>();
            i = 0;
            startTimeOut = System.currentTimeMillis();
            logger.debug("cellTemplates copy start: " + startTimeOut);
            for (CellTemplate cellTemplate : cellTemplateList) {
                i++;
                Long startTime = System.currentTimeMillis();
                logger.debug("celltemplate copy start: " + startTime);
                PeriodCellTemplate periodCellTemplate = new PeriodCellTemplate();
                CommonUtils.copyProperties(cellTemplate, periodCellTemplate);
                periodCellTemplate.setId(distributedIDService.nextId());
                periodCellTemplate.setPeriod(period);
                //                periodCellTemplate.setReportTemplateId(cellTemplate.getReportTemplateId());
//                periodCellTemplate.setRowIndex(cellTemplate.getRowIndex());
//                periodCellTemplate.setRowName(cellTemplate.getRowName());
//                periodCellTemplate.setColumnIndex(cellTemplate.getColumnIndex());
//                periodCellTemplate.setColumnName(cellTemplate.getColumnName());
//                periodCellTemplate.setComment(cellTemplate.getComment());
//                periodCellTemplate.setCreateTime(cellTemplate.getCreateTime());
//                periodCellTemplate.setUpdateTime(cellTemplate.getUpdateTime());
                periodCellTemplate.setCellTemplateId(cellTemplate.getId());
                //                periodCellTemplate.setDataType(cellTemplate.getDataType());
//                periodCellTemplate.setIsReadOnly(cellTemplate.getIsReadOnly());
//                periodCellTemplate.setCopyFromId(cellTemplate.getCopyFromId());
//                periodCellTemplate.setCreateBy(cellTemplate.getCreateBy());
//                periodCellTemplate.setUpdateBy(cellTemplate.getUpdateBy());
                periodCellTemplateList.add(periodCellTemplate);
                logger.debug("celltemplate copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
            }
            logger.debug("templates copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

            List<Long> cellTemplateIDList = periodCellTemplateList.stream().map(PeriodCellTemplate::getCellTemplateId).collect(Collectors.toList());
            CellTemplateConfigExample cellTemplateConfigExample = new CellTemplateConfigExample();
            cellTemplateConfigExample.createCriteria().andReportTemplateIdIn(templateIDList).andCellTemplateIdIn(cellTemplateIDList);
            List<CellTemplateConfig> cellTemplateConfigList = cellTemplateConfigMapper.selectByExample(cellTemplateConfigExample);
            List<PeriodCellTemplateConfig> periodCellTemplateConfigList = new ArrayList<>();
            i = 0;
            startTimeOut = System.currentTimeMillis();
            logger.debug("cellTemplateConfigs copy start:" + startTimeOut);
            for (CellTemplateConfig cellTemplateConfig : cellTemplateConfigList) {
                i++;
                Long startTime = System.currentTimeMillis();
                //logger.debug("cellTemplateConfig copy start: " + startTime);
                PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
                CommonUtils.copyProperties(cellTemplateConfig, periodCellTemplateConfig);
                periodCellTemplateConfig.setId(distributedIDService.nextId());
                periodCellTemplateConfig.setPeriod(period);
//                periodCellTemplateConfig.setCellTemplateId(cellTemplateConfig.getCellTemplateId());
//                periodCellTemplateConfig.setReportTemplateId(cellTemplateConfig.getReportTemplateId());
//                periodCellTemplateConfig.setDataSourceType(cellTemplateConfig.getDataSourceType());
//                periodCellTemplateConfig.setFormula(cellTemplateConfig.getFormula());
                periodCellTemplateConfig.setParsedFormula(StringUtils.EMPTY);
//                periodCellTemplateConfig.setFormulaDescription(cellTemplateConfig.getFormulaDescription());
//                periodCellTemplateConfig.setAccountCodes(cellTemplateConfig.getAccountCodes());
//                periodCellTemplateConfig.setInvoiceType(cellTemplateConfig.getInvoiceType());
//                periodCellTemplateConfig.setTaxRate(cellTemplateConfig.getTaxRate());
//                periodCellTemplateConfig.setInvoiceAmountType(cellTemplateConfig.getInvoiceAmountType());
//                periodCellTemplateConfig.setModelIds(cellTemplateConfig.getModelIds());
//                periodCellTemplateConfig.setCreateBy(cellTemplateConfig.getCreateBy());
//                periodCellTemplateConfig.setCreateTime(cellTemplateConfig.getCreateTime());
//                periodCellTemplateConfig.setUpdateBy(cellTemplateConfig.getUpdateBy());
//                periodCellTemplateConfig.setUpdateTime(cellTemplateConfig.getUpdateTime());
//                periodCellTemplateConfig.setInvoiceCategory(cellTemplateConfig.getInvoiceCategory());
//                periodCellTemplateConfig.setFormulaDataSource(cellTemplateConfig.getFormulaDataSource());
//                periodCellTemplateConfig.setValidation(cellTemplateConfig.getValidation());
                periodCellTemplateConfig.setParsedValidation(StringUtils.EMPTY);
//                periodCellTemplateConfig.setValidationDescription(cellTemplateConfig.getValidationDescription());
//                periodCellTemplateConfig.setVoucherKeyword(cellTemplateConfig.getVoucherKeyword());
                periodCellTemplateConfig.setCellTemplateConfigId(cellTemplateConfig.getId());
                periodCellTemplateConfig.setKeyValueParsedFormula(StringUtils.EMPTY);

                if (cellTemplateConfig.getFormula().startsWith("@")) {
                    //todo: get the keyvalue from table add to keyvalue parsed formula
                    KeyValueConfigExample keyValueConfigExample = new KeyValueConfigExample();
                    keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(cellTemplateConfig.getFormula(), "@"));
                    Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
                    if (keyValueConfig.isPresent()) {
                        periodCellTemplateConfig.setKeyValueParsedFormula(keyValueConfig.get().getFormula());
                    }
                }
                periodCellTemplateConfigList.add(periodCellTemplateConfig);
                //logger.debug("cellTemplateConfig copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
            }
            logger.debug("cellTemplateConfigs copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

            if (periodCellTemplateConfigList != null) {
                periodCellTemplateConfigList.forEach(a -> {
                    if (StringUtils.isNotBlank(a.getAccountCodes())) {
                        String[] acctCodes = a.getAccountCodes().split(",");
                        EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
                        enterpriseAccountExample.createCriteria().andStdCodeIn(Arrays.asList(acctCodes));
                        List<EnterpriseAccount> enterpriseAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
                        if (!enterpriseAccounts.isEmpty()) {
                            List<String> acctCodeList = enterpriseAccounts.stream().map(EnterpriseAccount::getCode).collect(Collectors.toList());
                            StringBuilder codes = new StringBuilder();
                            for (String code : acctCodeList) {
                                codes.append(code).append(",");
                            }
                            a.setAccountCodes(StringUtils.removeEnd(codes.toString(), ","));
                        }
                    }
                });
            }

            //todo: save config data to DB
            periodTemplateMapper.batchInsert(periodTemplateList);
            periodCellTemplateMapper.batchInsert(periodCellTemplateList);
            periodCellTemplateConfigMapper.batchInsert(periodCellTemplateConfigList);
        } else {
            result.setResult(true);
            result.setResultMsg("there is no templateGroup");
            return result;
        }
        result.setResult(true);
        return result;
    }

    @Override
    public OperationResultDto generateData(String projectId, EnumServiceType serviceType, Boolean ifDeleteManualDataSource,
                                           Integer periodParam, Integer reportType, Optional<String> generator) {
        OperationResultDto operationResultDto = new OperationResultDto();

        if (serviceType.equals(EnumServiceType.VAT) && (periodParam == null || periodParam <= 0)) {
            operationResultDto.setResultMsg("PeriodRequiredForVAT");
            return operationResultDto;
        }

        if (serviceType != EnumServiceType.VAT) {
            // 暂不支持非CIT/VAT service批量后端生成报表
            throw new NotSupportedException();
        }

        int period = periodParam;
        String serviceTypeStr = serviceType.getCode().toString();

        ProjectServiceTypeExample projectServiceTypeExample = new ProjectServiceTypeExample();
        projectServiceTypeExample.createCriteria().andServiceTypeIDEqualTo(serviceTypeStr).andProjectIDEqualTo(projectId);
        Optional<Long> templateGroupID = projectServiceTypeMapper.selectByExample(projectServiceTypeExample).stream()
                .map(ProjectServiceType::getTemplateGroupID).findFirst();
        if (templateGroupID == null) {
            operationResultDto.setResultMsg("TemplateGroupNotExist");
            return operationResultDto;
        }

        List<Long> templateIds;
        if (reportType != null) {
            PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
            periodTemplateExample.createCriteria()
                    .andTemplateGroupIdEqualTo(templateGroupID.get())
                    .andPeriodEqualTo(period)
                    .andIsActiveAssociationEqualTo(true)
                    .andReportTypeEqualTo(reportType);
            templateIds = periodTemplateMapper.selectByExample(periodTemplateExample).stream().map(PeriodTemplate::getId).collect(Collectors.toList());
        } else {
            PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
            periodTemplateExample.createCriteria().andTemplateGroupIdEqualTo(templateGroupID.get()).andPeriodEqualTo(period).andIsActiveAssociationEqualTo(true);
            templateIds = periodTemplateMapper.selectByExample(periodTemplateExample).stream().map(PeriodTemplate::getId).collect(Collectors.toList());
        }

        String rslt = reportGenerator.generateData(projectId, templateIds, ifDeleteManualDataSource, null, periodParam, generator);
        if (StringUtils.isBlank(rslt)) {
            operationResultDto.setResultMsg("ReportGenerateFailed!");
            return operationResultDto;
        }
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @Override
    public List<CellTemplateReferenceDto> getTemplateReferences(int period) {
        return new ArrayList<>();
    }

    @Override
    public OperationResultDto<ReportDataDto> getCellData(Long reportId) {
        OperationResultDto resultDto = new OperationResultDto();
        ReportDataDto dataDto = new ReportDataDto();

        Report report = reportMapper.selectByPrimaryKey(reportId);
        if (report == null) {
            resultDto.setResult(false);
            resultDto.setResultMsg("NoReport");
            return resultDto;
        }

        PeriodTemplateExample example = new PeriodTemplateExample();
        example.createCriteria().andTemplateIdEqualTo(report.getTemplateId()).andPeriodEqualTo(report.getPeriod());
        Optional<PeriodTemplate> reportTemplate = periodTemplateMapper.selectByExample(example).stream().findFirst();
        if (!reportTemplate.isPresent()) {
            resultDto.setResult(false);
            resultDto.setResultMsg("NoReportTemplate");
            return resultDto;
        }

        Project project = projectMapper.selectByPrimaryKey(report.getProjectId());
        if (project == null) {
            resultDto.setResult(false);
            resultDto.setResultMsg("NoProject");
            return resultDto;
        }

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria()
                .andTemplateGroupIdEqualTo(reportTemplate.get().getTemplateGroupId())
                .andPeriodEqualTo(report.getPeriod())
                .andIsActiveAssociationEqualTo(true);
        List<PeriodTemplate> reportTemplates = periodTemplateMapper.selectByExample(example);

        List<Long> templateIDs = new ArrayList<>();
        templateIDs.add(report.getTemplateId());
        List<CellCalcInfoDto> cellCfgList = reportGenerator.getCellCalcInfo(templateIDs, report.getPeriod());

        CellDataExample cellDataExample = new CellDataExample();
        cellDataExample.createCriteria().andReportIdEqualTo(reportId);

        List<CellData> currentCellDataList = cellDataMapper.selectByExample(cellDataExample);

        PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
        periodFormulaBlockExample.createCriteria().andPeriodEqualTo(report.getPeriod()).andReportIdEqualTo(reportId);
        List<PeriodFormulaBlock> formulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample);
        dataDto.setFormulaBlocks(formulaBlocks);
        //todo: 关键字数据源稍后再加
        // 获取通过KeyValueDataID关联的数据源，即公式数据源

        //todo: 模型数据源稍后再加
        // 获取通过ModelBackFillAmount关联的数据源，即模型数据源
        // TODO: Check模型回填数据源是否会有多对多的情况出现，如有需distinct

        //公式计算数据源 20180711 完成
        List<DataSourceExtendDto> manualDataSourceList = dataSourceMapper.getFormulaDataSource(reportId);
        //loop the datasource to add datasource detail information
        for (DataSourceExtendDto dseo : manualDataSourceList) {
            //只处理有明细数据的DataSource
            if (dseo.getType().equals(FormulaDataSourceType.InputInvoice.getCode())
                    || dseo.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())
                    || dseo.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                //todo: add json datasource detail @ here. not only itemValue
                List<String> itemValues = new ArrayList<>();
                itemValues.add(getDataSourceDetailList(dseo.getId()).getData());
                dseo.setItems(itemValues);
            } else if (dseo.getType().equals(FormulaDataSourceType.Report.getCode())) {
                List<String> itemValues = new ArrayList<>();
                //todo: get the datasource detail to add the items property
                DataSourceDetailExample dataSourceDetailExample = new DataSourceDetailExample();
                dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(dseo.getId());
                List<DataSourceDetail> dataSourceDetailList = SpringContextUtil.dataSourceDetailMapper.selectByExample(dataSourceDetailExample);
                dataSourceDetailList.forEach(a -> {
                    if (StringUtils.isNotBlank(a.getItemValue())) {
                        itemValues.add(a.getItemValue());
                    }
                });
                if (dataSourceDetailList.size() > 0 && dataSourceDetailList.get(0).getDataSourceType() != 6) {
                    dseo.setDataSourceType(convertType(dataSourceDetailList.get(0).getDataSourceType()));
                }
                dseo.setItems(itemValues);
            }
        }

        List<DataSourceDtoExtend> datasource = new ArrayList<>();
        manualDataSourceList.forEach(a -> {
            DataSourceDto dataSourceDto = new DataSourceDto();
            dataSourceDto.setId(a.getId().toString());
            dataSourceDto.setAmount(a.getAmount());
            dataSourceDto.setName(a.getName());
            dataSourceDto.setOperationType(a.getOperationType());
            dataSourceDto.setDescription(a.getDescription());
            dataSourceDto.setCreateTime(a.getCreateTime());
            if (a.getDataSourceType() != null && a.getDataSourceType() > 0) {
                dataSourceDto.setType(convetToFType(a.getDataSourceType()));
            } else {
                dataSourceDto.setType(a.getType());
            }
            dataSourceDto.setItems(a.getItems());
            dataSourceDto.setPeriod(a.getPeriod());
            dataSourceDto.setRowIndex(a.getRowIndex());
            dataSourceDto.setRowName(a.getRowName());
            dataSourceDto.setColumnIndex(a.getColumnIndex());
            dataSourceDto.setColumnName(a.getColumnName());
            dataSourceDto.setReportTemplateID(a.getReportTemplateID().toString());

            if (a.getType().equals(FormulaDataSourceType.Report.getCode())) {
                if (a.getItems().size() == 0) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.Formula.getCode());
                } else {
                    dataSourceDto.setDataSourceType(a.getDataSourceType());
                }
            } else if (a.getType().equals(FormulaDataSourceType.KeyInSource.getCode())) {
                dataSourceDto.setDataSourceType(CellDataSourceType.KeyIn.getCode());
            } else if (a.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
                dataSourceDto.setDataSourceType(CellDataSourceType.InputInvoice.getCode());
            } else if (a.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                dataSourceDto.setDataSourceType(CellDataSourceType.Voucher.getCode());
            } else {
                dataSourceDto.setDataSourceType(0);
            }

            DataSourceDtoExtend dataSourceDtoExtend = new DataSourceDtoExtend();
            dataSourceDtoExtend.setItem1(a.getCellTemplateID().toString());
            dataSourceDtoExtend.setItem2(dataSourceDto);

            datasource.add(dataSourceDtoExtend);
        });
        dataDto.setManualDataSources(datasource);

        List<CellDataDto> cellDataDtoList = new ArrayList<>();
        cellCfgList.forEach(a -> {
            CellDataDto cellDataDto = new CellDataDto();
            cellDataDto.setColumnIndex(a.getColumnIndex());
            cellDataDto.setColumnName(a.getColumnName());
            cellDataDto.setRowIndex(a.getRowIndex());
            cellDataDto.setRowName(a.getRowName());
            cellDataDto.setCellTemplateID(a.getCellTemplateID().toString());
            cellDataDto.setReportTemplateID(a.getReportTemplateID().toString());
            cellDataDto.setDataType(a.getDataType());
            cellDataDto.setReportID(reportId.toString());
            cellDataDto.setIsReadOnly(a.getIsReadOnly());
            Optional<CellData> celldata = currentCellDataList.stream()
                    .filter(x -> x.getCellTemplateId().toString().equals(a.getCellTemplateID())).findFirst();
            celldata.ifPresent(cellData -> cellDataDto.setCellID(cellData.getId().toString()));
            cellDataDto.setFormula(a.getFormula());
            celldata.ifPresent(cellData -> cellDataDto.setCellValue(cellData.getData()));
            celldata.ifPresent(cellData -> cellDataDto.setFormulaExp(cellData.getFormulaExp()));
            cellDataDto.setCellTemplateConfig(CellConfigTranslater.getConfigDto(a));
            List<DataSourceDtoExtend> entryList = datasource.stream()
                    .filter(y -> y.getItem1().equals(a.getCellTemplateID())).collect(Collectors.toList());
            List<DataSourceDto> dataSourceDtoList = new ArrayList<>();
            entryList.forEach(z -> dataSourceDtoList.add(z.getItem2()));
            cellDataDto.setDataSourceList(dataSourceDtoList);
            cellDataDtoList.add(cellDataDto);
        });

        ModifiedReportCellExample modifiedReportCellExample = new ModifiedReportCellExample();
        modifiedReportCellExample.createCriteria().andReportIdEqualTo(reportId);
        List<ModifiedReportCell> modifiedReportCellList = modifiedReportCellMapper.selectByExample(modifiedReportCellExample);
        modifiedReportCellList.forEach(mrc -> {
            Optional<CellDataDto> cdd = cellDataDtoList.stream()
                    .filter(cellDataDto -> cellDataDto.getReportID().equals(mrc.getReportId())
                            && cellDataDto.getRowIndex() == mrc.getRow()
                            && cellDataDto.getColumnIndex() == mrc.getCol()).findFirst();
            if (cdd.isPresent()) {
                if (!mrc.getOriginalValue().equals(mrc.getValue())) {
                    cdd.get().setModifiedReportCell(mrc);
                    cdd.get().setCellValue(mrc.getValue());
                    cdd.get().setIsModified(true);
                }
            }
        });
        dataDto.setCellData(cellDataDtoList);
        resultDto.setData(dataDto);
        resultDto.setResult(true);
        return resultDto;
    }

    private Integer convertType(Integer dataSourceType) {
        if (dataSourceType.equals(FormulaDataSourceDetailType.InputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.InputInvoice.getCode();
        } else {
            return CellDataSourceType.Formula.getCode();
        }
    }

    private Integer convetToFType(Integer cellDataSourceType) {
        if (cellDataSourceType.equals(CellDataSourceType.InputInvoice.getCode())) {
            return FormulaDataSourceType.InputInvoice.getCode();
        } else {
            return FormulaDataSourceType.Other.getCode();
        }
    }

    @Override
    public OperationResultDto<ReportDto> getReportByTemplate(Long templateId, Integer period) {
        OperationResultDto resultDto = new OperationResultDto();
        ReportExample example = new ReportExample();
        example.createCriteria().andTemplateIdEqualTo(templateId).andPeriodEqualTo(period);
        Optional<Report> report = reportMapper.selectByExample(example).stream().findFirst();
        if (report.isPresent()) {
            resultDto.setResult(true);
            ReportDto reportDto = new ReportDto();
            CommonUtils.copyProperties(report, reportDto);
            resultDto.setData(reportDto);
        } else {
            resultDto.setResult(false);
            resultDto.setResultMsg("can not find report");
        }
        return resultDto;
    }

    @Override
    public OperationResultDto<PeriodCellTemplateConfig> getCellTemplateConfig(Long reportTemplateId, Integer periodParam, int rowIndex, int columnIndex) {
        OperationResultDto resultDto = new OperationResultDto();
        int period = periodParam != null ? periodParam : 0;

        if (reportTemplateId == null || reportTemplateId == 0L) {
            resultDto.setResult(false);
            resultDto.setResultMsg("report template id is invalid");
            return resultDto;
        }

        PeriodCellTemplateConfig periodCellTemplateConfig = periodCellTemplateConfigMapper.getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex);
        if (periodCellTemplateConfig == null) {
            resultDto.setResult(false);
            resultDto.setResultMsg("no config for this template!");
            return resultDto;
        } else {
            resultDto.setResult(true);
            resultDto.setData(periodCellTemplateConfig);
            return resultDto;
        }
    }

    @Override
    public OperationResultDto<List<VatEnterpriseAccount>> getStdAccountByIndustry(String industryID) {
        OperationResultDto resultDto = new OperationResultDto();
        VatEnterpriseAccountExample example = new VatEnterpriseAccountExample();
        example.createCriteria().andIsActiveEqualTo(true);
        vatEnterpriseAccountMapper.selectByExample(example);
        List<VatEnterpriseAccount> vatEnterpriseAccounts = vatEnterpriseAccountMapper.selectByExample(example);
        if (vatEnterpriseAccounts == null) {
            resultDto.setResult(false);
            resultDto.setResultMsg("there is no data");
            return resultDto;
        } else {
            resultDto.setResult(true);
            resultDto.setData(vatEnterpriseAccounts);
            return resultDto;
        }
    }

    @Override
    public OperationResultDto<List<VatEnterpriseAccountResultDto>> getCellAccountRange(Long reportTemplateId, Integer periodParam, int rowIndex, int columnIndex) {
        OperationResultDto resultDto = new OperationResultDto();

        int period = periodParam == null ? 0 : periodParam;

        PeriodCellTemplateConfig periodCellTemplateConfig =
                periodCellTemplateConfigMapper.getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex);
        if (periodCellTemplateConfig == null || StringUtils.isBlank(periodCellTemplateConfig.getAccountCodes())) {
            resultDto.setResult(false);
            return resultDto;
        }

        VatEnterpriseAccountExample example = new VatEnterpriseAccountExample();
        example.createCriteria().andAcctCodeIn(Arrays.asList(periodCellTemplateConfig.getAccountCodes().split(",")));
        List<VatEnterpriseAccount> vatEnterpriseAccounts = vatEnterpriseAccountMapper.selectByExample(example);
        List<VatEnterpriseAccountResultDto> vatEnterpriseAccountResultDtos = new ArrayList<>();
        for (VatEnterpriseAccount vatEnterpriseAccount : vatEnterpriseAccounts) {
            VatEnterpriseAccountResultDto vatEnterpriseAccountResultDto = new VatEnterpriseAccountResultDto();
            vatEnterpriseAccountResultDto.setPeriodCellTemplateConfigId(periodCellTemplateConfig.getCellTemplateConfigId());
            vatEnterpriseAccountResultDto.setAcctCode(vatEnterpriseAccount.getAcctCode());
            vatEnterpriseAccountResultDto.setAcctPropName(vatEnterpriseAccount.getName());
            vatEnterpriseAccountResultDto.setDirectionName(vatEnterpriseAccount.getDirection() == 1 ? "借方" : "贷方");
            vatEnterpriseAccountResultDto.setAcctPropName(getAcctProp(vatEnterpriseAccount.getAcctProp()));
            vatEnterpriseAccountResultDtos.add(vatEnterpriseAccountResultDto);
        }

        resultDto.setData(vatEnterpriseAccountResultDtos);
        resultDto.setResult(true);
        return resultDto;
    }

    @Override
    public OperationResultDto addCellManualDataSource(ManualDataSourceDto data) {
        OperationResultDto operationResultDto = new OperationResultDto();

        if (data.getCellId() == null) {
            //todo: insert celldata for manual datasource
            CellData cellData = new CellData();
            cellData.setId(distributedIDService.nextId());
            cellData.setReportId(data.getReportId());
            cellData.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
            cellData.setData(data.getAmount().toString());
            cellData.setFormulaExp(data.getAmount().toString());
            cellData.setCreateBy("admin");
            cellData.setCreateTime(new Date());
            cellData.setUpdateBy("admin");
            cellData.setUpdateTime(new Date());
            cellDataMapper.insertSelective(cellData);
            data.setCellId(cellData.getId());
        } else {
            CellData cellData = cellDataMapper.selectByPrimaryKey(data.getCellId());
            if (cellData.getData() != data.getAmount().toString()) {
                cellData.setData(data.getAmount().toString());
                cellDataMapper.updateByPrimaryKeySelective(cellData);
            }
        }

        List<DataSourceExtendDto> dataSourceExtendDtos = dataSourceMapper.getManualDataSource(data.getCellId());

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andCellTemplateIdEqualTo(Long.parseLong(data.getCellTemplateId())).andPeriodEqualTo(data.getPeriod());
        Optional<PeriodCellTemplate> periodCellTemplate = periodCellTemplateMapper.selectByExample(periodCellTemplateExample).stream().findFirst();

        BigDecimal originalAmount = new BigDecimal("0");
        DataSource dataSourceModel = null;
        if (dataSourceExtendDtos.size() > 0) {
            dataSourceModel = dataSourceExtendDtos.get(0).getDataSource();
            updateCellValueForDataSourceChange(dataSourceModel, data.getAmount());
            originalAmount = dataSourceModel.getAmount() != null ? dataSourceModel.getAmount() : new BigDecimal("0");
            dataSourceModel.setName(data.getName());
            dataSourceModel.setDescription(data.getDescription());
            dataSourceModel.setAmount(data.getAmount());
            dataSourceModel.setUpdateBy("Admin");
            dataSourceModel.setUpdateTime(new Date());
            dataSourceMapper.updateByPrimaryKeySelective(dataSourceModel);
        } else {
            dataSourceModel = new DataSource();
            Long cellDataSourceID = distributedIDService.nextId();
            dataSourceModel.setId(cellDataSourceID);
            dataSourceModel.setType(FormulaDataSourceType.KeyInSource.getCode());
            dataSourceModel.setName(data.getName());
            dataSourceModel.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
            dataSourceModel.setDescription(data.getDescription());
            dataSourceModel.setAmount(data.getAmount());
            if (periodCellTemplate.isPresent()) {
                dataSourceModel.setRowIndex(periodCellTemplate.get().getRowIndex());
                dataSourceModel.setColumnIndex(periodCellTemplate.get().getColumnIndex());
            }
            dataSourceModel.setCreateTime(new Date());
            dataSourceModel.setUpdateTime(new Date());
            dataSourceModel.setCreateBy("admin");
            dataSourceModel.setUpdateBy("admin");
            dataSourceModel.setPeriod(data.getPeriod());

            CellDataSource cellDataSource = new CellDataSource();
            cellDataSource.setId(distributedIDService.nextId());
            cellDataSource.setCellDataId(data.getCellId());
            cellDataSource.setDataSourceId(cellDataSourceID);
            cellDataSource.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
            cellDataSource.setCreateTime(new Date());
            cellDataSource.setUpdateTime(new Date());
            updateCellValueForDataSourceChange(cellDataSource.getCellDataId(), new BigDecimal("0"), data.getAmount());
            dataSourceMapper.insertSelective(dataSourceModel);
            cellDataSourceMapper.insertSelective(cellDataSource);
            dataSourceExtendDtos = dataSourceMapper.getManualDataSource(data.getCellId());
        }

        //todo: update the reference cell's data, such as if formula is ND(1)+A2,
        //todo: if A2 is changed manually, ND(1)+A2 result to changed, and datasource of A2 should be updated
        //1. use celltemplateid and period get template id, the change should be in the same sheet.if not in the same sheet, the report should be recalculate, can't changed in time.
        //2. use the celltemplateid get col and row number, get the cell name such as A2,B3 etc.
        String cellName = StringUtils.EMPTY;
        Long templateId = 0L;
        if (periodCellTemplate.isPresent()) {
            Integer colNum = periodCellTemplate.get().getColumnIndex();
            Integer rowNum = periodCellTemplate.get().getRowIndex() + 1;
            cellName = FormulaHelper.excelColIndexToStr(colNum + 1) + rowNum.toString();
            templateId = periodCellTemplate.get().getReportTemplateId();
        }
        //3. use the template id get the reportid, use the reportid get the celltemplateids, then use the celltemplateids and cell name get the datasource which should be updated.
        ReportExample reportExample = new ReportExample();
        reportExample.createCriteria().andTemplateIdEqualTo(templateId).andPeriodEqualTo(data.getPeriod());
        Optional<Report> report = reportMapper.selectByExample(reportExample).stream().findFirst();
        if (report.isPresent()) {
            Long reportId = report.get().getId();
            CellDataExample cellDataExample = new CellDataExample();
            cellDataExample.createCriteria().andReportIdEqualTo(reportId);
            List<CellData> cellDataList = cellDataMapper.selectByExample(cellDataExample);
            for (CellData cellData : cellDataList) {
                if (cellData.getId() != data.getCellId() && cellData.getFormulaExp().contains(cellName)) {
                    CellDataSourceExample cellDataSourceExample = new CellDataSourceExample();
                    cellDataSourceExample.createCriteria().andCellDataIdEqualTo(cellData.getId()).andCellTemplateIdEqualTo(cellData.getCellTemplateId());
                    List<CellDataSource> cellDataSourceList = cellDataSourceMapper.selectByExample(cellDataSourceExample);
                    for (CellDataSource cellDataSource : cellDataSourceList) {
                        Long dataSourceId = cellDataSource.getDataSourceId();
                        DataSource dataSource = dataSourceMapper.selectByPrimaryKey(dataSourceId);
                        if (dataSource.getDescription().equals(cellName)) {
                            BigDecimal changeValue = data.getAmount().subtract(dataSource.getAmount());
                            dataSource.setAmount(data.getAmount());
                            dataSource.setUpdateBy("admin");
                            dataSource.setUpdateTime(new Date());
                            dataSourceMapper.updateByPrimaryKeySelective(dataSource);
                            if (!cellData.getData().equals("#VALUE!")) {
                                cellData.setData(new BigDecimal(cellData.getData()).add(changeValue).toString());
                            } else {
                                cellData.setData(new BigDecimal("0.0").add(changeValue).toString());
                            }
                            //cellData.setFormulaExp(cellData.getData());
                            cellData.setUpdateTime(new Date());
                            cellDataMapper.updateByPrimaryKeySelective(cellData);
                        }
                    }
                }
            }
        }
        //4. before you do the setp3 ,you should get the changed value of datasource. then update the celldata use this change. just add the change value to celldata amount.
        //5. then save the datasource and save the celldata.
        data.setId(dataSourceExtendDtos.get(0).getId());
        operationResultDto.setResult(true);
        operationResultDto.setData(data);
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> addDataSource(DataSourceDto datasourceDto) {
        OperationResultDto operationResultDto = new OperationResultDto();

        CellData cellData = cellDataMapper.selectByPrimaryKey(Long.parseLong(datasourceDto.getCellDataID() != null ? datasourceDto.getCellDataID() : "0"));
        if (cellData == null) {
            operationResultDto.setResult(false);
            operationResultDto.setResultMsg("NoCellData");
            return operationResultDto;
        }

        DataSource dataSource = new DataSource();
        CommonUtils.copyProperties(datasourceDto, dataSource);
        dataSource.setType(datasourceDto.getDataSourceType());
        if (dataSource.getId() == null || dataSource.getId() == 0L) {
            dataSource.setId(distributedIDService.nextId());
            dataSource.setCreateTime(new Date());
            dataSource.setCellTemplateId(Long.parseLong(datasourceDto.getCellTemplateID()));
            dataSource.setUpdateTime(new Date());
            dataSource.setUpdateBy(datasourceDto.getUpdater());
            dataSource.setCreateBy(datasourceDto.getCreator());
        }
        dataSource.setCreateTime(new Date());
        dataSourceMapper.insertSelective(dataSource);

        CellDataSource cellDataSource = new CellDataSource();
        cellDataSource.setId(distributedIDService.nextId());
        cellDataSource.setCellDataId(cellData.getId());
        cellDataSource.setCellTemplateId(cellData.getCellTemplateId());
        cellDataSource.setOperationType(datasourceDto.getOperationType());
        cellDataSource.setDataSourceId(dataSource.getId());
        cellDataSource.setCreateTime(new Date());
        cellDataSource.setUpdateTime(new Date());

        cellDataSourceMapper.insertSelective(cellDataSource);

        operationResultDto.setData(dataSource.getId().toString());
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> getAllDataItems(Integer dataSourceType) {
        OperationResultDto operationResultDto = new OperationResultDto();
        CellDataSourceType cellDataSourceType = CellDataSourceType.getDSType(dataSourceType);
        String result = StringUtils.EMPTY;
        switch (cellDataSourceType) {
            case Voucher:
                List<CellVoucherDto> cellVoucherDtos = voucherMapper.selectVoucherWithEnterpriseAccount();
                result = JSON.toJSONString(cellVoucherDtos);
                break;
            case OutputInvoice:
                List<OutputInvoiceDataSourceDto> outputInvoiceDataSourceDtos = outputVATInvoiceMapper.selectOutputInvoiceAndItem();
                result = JSON.toJSONString(outputInvoiceDataSourceDtos);
                break;
            case InputInvoice:
                List<CellInvoiceDto> cellInvoiceDtos = inputVATInvoiceMapper.selectInputInvoiceWithItem();
                result = JSON.toJSONString(cellInvoiceDtos);
                break;
            case CustomInvoice:
                List<CustomsInvoice> customsInvoices = customsInvoiceMapper.selectByExample(null);
                result = JSON.toJSONString(customsInvoices);
                break;
            case Formula:
                break;
            case KeyIn:
                break;
            case RelatedModel:
                break;
            case SapDaily:
                break;
            default:
                break;
        }
        operationResultDto.setData(result);
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @Override
    public OperationResultDto addDataSourceItems(DataSourceDto dataSource) {
        OperationResultDto operationResultDto = new OperationResultDto();

        if (dataSource.getItems() == null || dataSource.getItems().isEmpty()) {
            operationResultDto.setResultMsg("NoData");
            operationResultDto.setResult(false);
            return operationResultDto;
        }

        if (dataSource == null || StringUtils.isBlank(dataSource.getId())) {
            operationResultDto.setResultMsg("DataSourceInvalid");
            operationResultDto.setResult(false);
            return operationResultDto;
        }

        DataSource datasourceEntity = dataSourceMapper.selectByPrimaryKey(Long.parseLong(dataSource.getId()));
        if (datasourceEntity == null) {
            operationResultDto.setResultMsg("DatasourceInvalid");
            operationResultDto.setResult(false);
            return operationResultDto;
        }
        updateCellValueForDataSourceChange(datasourceEntity, dataSource.getAmount());
        dataSourceMapper.updateByPrimaryKeySelective(datasourceEntity);
        //todo:MarkProjectModelDirty(dataSource.projectID, dataSource.serviceTypeID, dataSource.Updater);

        List<DataSourceDetail> dataSourceItems = new ArrayList<>();
        dataSource.getItems().forEach(a -> {
            DataSourceDetail dataSourceDetail = new DataSourceDetail();
            dataSourceDetail.setId(distributedIDService.nextId());
            dataSourceDetail.setDataSourceId(Long.parseLong(dataSource.getId()));
            dataSourceDetail.setDataSourceType(dataSource.getDataSourceType());
            dataSourceDetail.setItemValue(a);
            dataSourceItems.add(dataSourceDetail);
        });

        DataSourceDetailExample example = new DataSourceDetailExample();
        example.createCriteria().andDataSourceIdEqualTo(Long.parseLong(dataSource.getId()));
        List<DataSourceDetail> dataSourceDetailList = dataSourceDetailMapper.selectByExample(example);
        //todo: TBC tomorrow
        dataSourceDetailList.forEach(a -> {
            dataSourceDetailMapper.deleteByPrimaryKey(a.getId());
        });

        dataSourceItems.forEach(a -> {
            dataSourceDetailMapper.insertSelective(a);
        });

        operationResultDto.setResult(true);
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> getDataSourceDetailList(Long datasSourceId) {
        OperationResultDto<String> operationResultDto = new OperationResultDto<>();
        DataSource dataSource = dataSourceMapper.selectByPrimaryKey(datasSourceId);
        String itemStrValue = StringUtils.EMPTY;

        if (dataSource.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
            DataSourceDetailExample dataSourceDetailExample = new DataSourceDetailExample();
            dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(datasSourceId).andDataSourceTypeEqualTo(CellDataSourceType.InputInvoice.getCode());
            List<DataSourceDetail> dataSourceDetails = dataSourceDetailMapper.selectByExample(dataSourceDetailExample);
            List<InputVATInvoiceItemExtendDto> inputVATInvoiceItemExtendDtos = new ArrayList<>();
            for (DataSourceDetail dataSourceDetail : dataSourceDetails) {
                if (StringUtils.isNotBlank(dataSourceDetail.getItemValue())) {
                    InputVATInvoiceItemExtendDto inputVATInvoiceItemExtendDto = inputVATInvoiceMapper.selectInvoiceExtendDto(dataSourceDetail.getItemValue());
                    inputVATInvoiceItemExtendDtos.add(inputVATInvoiceItemExtendDto);
                }
            }
            itemStrValue = JSON.toJSONString(inputVATInvoiceItemExtendDtos);
        } else if (dataSource.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
            DataSourceDetailExample dataSourceDetailExample = new DataSourceDetailExample();
            dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(datasSourceId).andDataSourceTypeEqualTo(CellDataSourceType.Voucher.getCode());
            List<DataSourceDetail> dataSourceDetails = dataSourceDetailMapper.selectByExample(dataSourceDetailExample);
            List<Voucher> vouchers = new ArrayList<>();
            for (DataSourceDetail dataSourceDetail : dataSourceDetails) {
                if (StringUtils.isNotBlank(dataSourceDetail.getItemValue())) {
                    Voucher voucher = voucherMapper.selectByPrimaryKey(dataSourceDetail.getItemValue());
                    vouchers.add(voucher);
                }
            }
            itemStrValue = JSON.toJSONString(vouchers);
        } else {
            itemStrValue = StringUtils.EMPTY;
        }

        operationResultDto.setData(itemStrValue);
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    private void updateCellValueForDataSourceChange(Long cellDataID, BigDecimal originalAmount, BigDecimal newAmount) {
        BigDecimal aValue = newAmount != null ? newAmount : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        CellData cellData = cellDataMapper.selectByPrimaryKey(cellDataID);
        BigDecimal cellVal = new BigDecimal("0");
        try {
            if (cellData != null) {
                cellVal = new BigDecimal(cellData.getData());
                Integer cellDataType = periodCellTemplateMapper
                        .getDataType(cellData.getCellTemplateId()
                                , cellData.getReportId());
                cellData.setData(FormulaHelper.roundValue(cellVal.add(amountChange)
                        , KeyValueConfigResultType.values()[cellDataType], false, null).toString());
            }
        } catch (NumberFormatException ex) {
        }
    }

    private void updateCellValueForDataSourceChange(DataSource dataSourceEntity, BigDecimal amount) {
        BigDecimal originalAmount = dataSourceEntity.getAmount();
        dataSourceEntity.setAmount(amount);
        BigDecimal aValue = dataSourceEntity.getAmount() != null ? dataSourceEntity.getAmount() : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        DataSourceCellDataDto dataSourceCellDataDto =
                dataSourceMapper.getManualDataSource2(dataSourceEntity.getId());
        BigDecimal cellVal = new BigDecimal("0");
        if (dataSourceCellDataDto != null && dataSourceCellDataDto.getCellData().getData() != null) {
            if (dataSourceCellDataDto.getCellDataSource().getOperationType().equals(EnumOperationType.Sub)) {
                amountChange = BigDecimal.ZERO.subtract(amountChange);
            }

            try {
                cellVal = new BigDecimal(dataSourceCellDataDto.getCellData().getData());
                if (StringUtils.isNotBlank(dataSourceCellDataDto.getCellData().getData())) {
                    Integer cellDataType = periodCellTemplateMapper
                            .getDataType(dataSourceCellDataDto.getCellData().getCellTemplateId()
                                    , dataSourceCellDataDto.getCellData().getReportId());
                    dataSourceCellDataDto.getCellData().setData(
                            FormulaHelper.roundValue(cellVal.add(amountChange),
                                    KeyValueConfigResultType.values()[cellDataType],
                                    false,
                                    null).toString());
                }
            } catch (NumberFormatException ex) {
            }
        }
    }

    private String getAcctProp(Integer acctProp) {
        switch (acctProp) {
            case 1:
                return "资产";
            case 2:
                return "负债";
            case 3:
                return "共同";
            case 4:
                return "权益";
            case 5:
                return "成本";
            case 6:
                return "损益";
            default:
                return "";
        }
    }
}
