﻿commonModule.directive('vatReportSheet', ['$rootScope', '$log', '$timeout', '$q', 'templateService', 'enums', 'keyValueDataService', 'spreadJsTipService',
    '$interval', 'vatSessionService',
    function ($rootScope, $log, $timeout, $q, templateService, enums, keyValueDataService, spreadJsTipService, $interval, vatSessionService) {
        'use strict';
        $log.debug('vatReportSheet.ctor()...');

        return {
            restrict: 'E',
            template: '<div class="report" style="height: 100%;"></div>',
            replace: true,
            scope: {
                templateId: '=',
                spread: '=',
                reportSource: '=',
                formulaBlocks: '=',
                manualDataSources: '=',
                isReadOnly: '=',
                serviceType: '=',
                initRow: '=?',
                initCol: '=?',
                reportApi: '=?' // In: onCellClick, onCellDoubleClick; Out: refreshReport
            },
            link: function (scope, element) {
                $log.debug('vatReportSheet.link()...');

                // Register api functions in internalApi
                // If selectorApi passed is not undefined, we will use it as internalApi
                scope.internalApi = scope.reportApi || {};

                scope.projectYear = vatSessionService.year;
                scope.projectPeriod = scope.serviceType === enums.serviceType.VAT ? vatSessionService.month : 0;

                scope.$watch('templateId', function (newVal, oldValue) {
                    if (newVal !== oldValue) {
                        loadSheet(newVal);
                    }
                });

                scope.$watchGroup(['reportSource', 'formulaBlocks', 'manualDataSources', 'templateId', 'hasLoadSpread'], function (newVal, oldValue) {
                    if (scope.templateId && scope.reportSource && scope.hasLoadSpread) {
                        setData();
                    }
                });

                // 首次进入页面时，通过promise模式保证在加载完成后再调用locateCell以focus在特定单元格
                // 如果是在当前页面要focus在特定单元格，通过watch行列号来完成
                scope.$watchGroup(['initRow', 'initCol'], function (newVal, oldVal) {
                    if (_.isNumber(scope.initRow) && _.isNumber(scope.initCol)) {
                        var spreadCtrl = getSpreadControl();
                        if (spreadCtrl) {
                            locateCell(spreadCtrl);
                        }

                        spreadCtrl = null;
                    }
                });

                var locateCell = function (spread, forceCloseModal) {
                    if (_.isNumber(scope.initRow) && _.isNumber(scope.initCol)) {
                        var sheet;
                        if (constant.regesterInformation.active) {
                            sheet = spread.getActiveSheet();
                        }
                        else {
                            sheet = spread.getSheet(1);
                        }

                        //var sheet = spread.getActiveSheet();
                        //var sheet = spread.getSheet(1);

                        sheet.setSelection(scope.initRow, scope.initCol, 1, 1);
                        if (_.isFunction(scope.internalApi.onCellClick) && !forceCloseModal) {
                            scope.internalApi.onCellClick({
                                'args': {row: scope.initRow, col: scope.initCol},
                                'data': JSON.parse(sheet.getTag(scope.initRow, scope.initCol))
                            });
                        }

                        $log.debug(sheet.getTag(scope.initRow, scope.initCol));

                        scope.initRow = null;
                        scope.initCol = null;
                        sheet = null;

                        return $q.when();
                    }
                };

                var loadSheet = function (templateId) {
                    return templateService.getTemplateJson(templateId).then(function (reportSpread) {
                        var spreadCtrl = getSpreadControl();
                        if (spreadCtrl) {
                            spreadCtrl.destroy();
                        }

                        spreadCtrl = null;

                        if (!_.isEmpty(reportSpread)) {
                            initSpreadExcel(reportSpread).then(function (spread) {
                                return locateCell(spread, true);
                            });
                        }
                    }, function (data) {
                        $log.info(data);
                    });
                };

                var initSpreadExcel = function (reportSpread) {

                    var spread = new GC.Spread.Sheets.Workbook(getSpreadControlElement());
                    spread.suspendPaint();
                    spread.fromJSON(reportSpread);

                    spread.options.showVerticalScrollbar = true;
                    spread.options.showHorizontalScrollbar = true;
                    spread.options.scrollbarMaxAlign = true;
                    spread.options.scrollbarShowMax = true;
                    spread.options.tabNavigationVisible = true;
                    if (constant.regesterInformation.active) {
                        spread.options.tabEditable = false;
                        spread.options.tabStripVisible = false;
                        spread.options.newTabVisible = false;
                    }
                    spread.options.allowUndo = false;
                    spread.options.allowUserResize = false;
                    spread.options.allowUserDragDrop = false;
                    spread.options.allowUserDragFill = false;
                    spread.options.allowUserEditFormula = false;
                    spread.options.allowContextMenu = false;
                    initInterFunction(spread);

                    //todo:注册之后这里去掉注释
                    //var sheet = spread.getActiveSheet();
                    var sheet = spread.getSheet(1);

                    if (sheet != null) {
                        sheet.options.rowHeaderVisible = true;
                        sheet.options.colHeaderVisible = true;
                        sheet.options.gridline.showVerticalGridline = false;
                        sheet.options.gridline.showHorizontalGridline = false;
                        sheet.options.isProtected = true;

                        if (scope.isReadOnly) {
                            setEditable(spread, true, false);
                        }

                        //setCellFormula(template.id, sheet);
                        //给单元格增加双击事件
                        sheet.bind(GC.Spread.Sheets.Events.CellDoubleClick, function (sender, args) {
                            scope.$apply(function () {
                                if (_.isFunction(scope.internalApi.onCellDoubleClick) && args.sheet.getTag(args.row, args.col)) {
                                    scope.internalApi.onCellDoubleClick({
                                        'sender': sender,
                                        'args': args,
                                        'data': JSON.parse(args.sheet.getTag(args.row, args.col))
                                    });
                                    $log.debug(args.sheet.getTag(args.row, args.col));
                                }
                            });
                        });

                        //给单元格增加单击事件
                        sheet.bind(GC.Spread.Sheets.Events.CellClick, function (sender, args) {
                            scope.$apply(function () {
                                if (window.event.ctrlKey) {
                                    var data = JSON.parse(args.sheet.getTag(args.row, args.col));
                                    if (!_.isEmpty(data) && !_.isEmpty(data.dataSourceList)) {
                                        var reports = _.chain(data.dataSourceList)
                                            .where({type: enums.formulaDataSourceType.Report})
                                            .map(function (x) {
                                                return {
                                                    reportCode: x.reportCode,
                                                    year: x.year,
                                                    period: x.period
                                                };
                                            }).uniq(function (x) {
                                                return x.reportCode + ',' + x.year + ',' + x.period;
                                            }).value();
                                        if (reports.length === 1) {
                                            var drillDownArgs = _.findWhere(data.dataSourceList, {type: enums.formulaDataSourceType.Report});
                                            if (drillDownArgs.period === scope.projectPeriod && drillDownArgs.year === scope.projectYear) {
                                                drillDownArgs = {
                                                    reportID: drillDownArgs.reportID,
                                                    reportTemplateID: drillDownArgs.reportTemplateID,
                                                    reportName: drillDownArgs.reportName,
                                                    reportCode: drillDownArgs.reportCode,
                                                    cellName: PWC.numToExcelChar(drillDownArgs.rowIndex, drillDownArgs.columnIndex)
                                                };
                                                scope.$emit(enums.vatEvent.reportDrillDown, drillDownArgs);
                                            }
                                        }
                                    }
                                }
                                else if (_.isFunction(scope.internalApi.onCellClick) && args.sheet.getTag(args.row, args.col)) {
                                    scope.internalApi.onCellClick({
                                        'sender': sender,
                                        'args': args,
                                        'data': JSON.parse(args.sheet.getTag(args.row, args.col))
                                    });
                                    $log.debug(args.sheet.getTag(args.row, args.col));
                                }
                            });
                        });

                        // 编辑完成后记录更新的单元格，适用于isReadOnly为false时
                        sheet.bind(GC.Spread.Sheets.Events.EditEnded, function (sender, args) {
                            scope.$apply(function () {
                                var cell = sheet.getCell(args.row, args.col);
                                var newVal = cell.value();
                                var cellInfo = _.find(scope.reportSource, {rowIndex: args.row, columnIndex: args.col});
                                if (cellInfo) {
                                    cellInfo.value = newVal;
                                    cellInfo.isDirty = true;
                                }
                                $log.debug("Edit cell (" + args.row + ", " + args.col + ")");
                            });
                        });

                        sheet.clearSelection();
                    }
                    spread.resumePaint();
                    // spread.isPaintSuspended(false);

                    scope.spread = spread;
                    scope.hasLoadSpread = true;

                    return $q.when(spread);
                };

                var setEditable = function (spread, isReadOnly, ifSuspend) {

                    //todo:注册之后这里去掉注释
                    var sheet;
                    if (constant.regesterInformation.active) {
                        sheet = spread.getActiveSheet();
                    }
                    else {
                        sheet = spread.getSheet(1);
                    }

                    if (sheet != null) {
                        if (ifSuspend) {
                            spread.suspendPaint();
                        }
                    }

                    // isReadOnly为true时，设置整个sheet不可编辑
                    if (isReadOnly) {
                        for (var row = 0; row < sheet.getRowCount(); row++) {
                            for (var column = 0; column < sheet.getColumnCount(); column++) {
                                try {
                                    sheet.getCell(row, column).locked(true);
                                }
                                catch (e) {
                                }
                            }
                        }
                    }
                    else {
                        // isReadOnly为false时，设置无配置单元格不可编辑，有配置单元格可编辑
                        for (var i = 0; i < scope.reportSource.length; i++) {
                            var cell = scope.reportSource[i];
                            sheet.getCell(cell.rowIndex, cell.columnIndex).locked(false);
                        }

                        for (var x = 0; x < sheet.getRowCount(); x++) {
                            for (var y = 0; y < sheet.getColumnCount(); y++) {
                                var idx = _.findIndex(scope.reportSource, {rowIndex: x, columnIndex: y});
                                if (idx < 0) {
                                    sheet.getCell(x, y).locked(true);
                                }
                            }
                        }
                    }

                    sheet.options.isProtected = true;
                    sheet.options.protectionOptions = {
                        allowEditObjects: false
                    };
                    if (sheet != null) {
                        if (ifSuspend) {
                            spread.resumePaint();
                        }
                    }
                };

                // 每个单元格大致由以下部分构成：_SumAll(_Inter(...)+_Manual(...)+C1)
                // _Inter: IronPython公式计算结果
                // _Manual: 手工数据源值
                // C1: 类似的Excel公式，通过SpreadJS公式引擎计算
                var initInterFunction = function (spread) {
                    // var interFunc = function () {
                    // };
                    // interFunc.prototype = new GcSpread.Sheets.Calc.Functions.Function("_Inter", 1, 1);
                    // interFunc.prototype.evaluate = function (args) {
                    //     if (args.length !== 1 || isNaN(parseInt(args))) {
                    //         return "#VALUE!";
                    //     }
                    //
                    //     var idx = args[0];
                    //     var b = _.findWhere(scope.formulaBlocks, {index: idx});
                    //     if (b) {
                    //         var rtn = b.data;
                    //         rtn = PWC.tryParseStringToNum(rtn);
                    //
                    //         return rtn;
                    //     }
                    //
                    //     return null;
                    // };
                    var activeSheet = spread.getSheet(1);
                    var manualFunc = function () {
                    };
                    manualFunc.prototype = new GC.Spread.CalcEngine.Functions.Function("_Manual", 1, 1);
                    manualFunc.prototype.evaluate = function (args) {
                        if (!_.isString(args)) {
                            return "#VALUE!";
                        }

                        //var cellTemplateID = Number(args);
                        var ds = _.where(scope.manualDataSources, {item1: args});
                        if (!_.isEmpty(ds)) {
                            return _.reduce(ds, function (memo, x) {
                                if (x.item2.dataSourceType === 6) {
                                    var rtn = x.item2.amount;
                                    rtn = PWC.tryParseStringToNum(rtn);

                                    // 如果是数值数据源，只有OperationType为2时，该数据源的数值应被减去；其他情况下应被加上
                                    if (_.isNumber(rtn) && !isNaN(rtn)) {
                                        var operationType = x.item2.operationType;
                                        rtn = operationType === 2 ? (-rtn) : rtn;
                                    }

                                    return memo + rtn;
                                }
                                else {
                                    return memo;
                                }
                            }, 0);
                        }

                        return 0;
                    };

                    //var sumAllFunc = function () { };
                    //sumAllFunc.prototype = new GcSpread.Sheets.Calc.Functions.Function("_SumAll", 1, 255);
                    //sumAllFunc.prototype.evaluate = function (args) {
                    //    if (args.length < 1) {
                    //        return "";
                    //    }

                    //    var rtn = args[0];
                    //    if (args.length === 1 && (_.isNull(rtn) || _.isUndefined(rtn))) {
                    //        rtn = "";
                    //    }
                    //    else {
                    //        // 下面这种奇怪的写法是为了保证在reduce过程中累计值的类型正确
                    //        rtn = _.reduce(args.slice(1), function (memo, x) {
                    //            return memo + x;
                    //        }, rtn);
                    //    }

                    //    return rtn;
                    //};

                    spread.clearCustomFunctions();
                    //spread.addCustomFunction(new interFunc());
                    activeSheet.addCustomFunction(new manualFunc());
                    //spread.addCustomFunction(new sumAllFunc());
                };

                var layoutChangedFunc = scope.$on(enums.vatEvent.layoutChanged, function () {
                    repaintSpread();
                });

                // TODO: 手工修改单元格值之后，更新单元格状态
                var cellValueModifiedFunc = $rootScope.$on('cellValueModified', function (event, args) {
                    var modifiedReportCell = args.modifiedReportCell;
                    var cell = scope.reportSource.filter(function (data) {
                        return data.rowIndex === modifiedReportCell.row && data.columnIndex === modifiedReportCell.col;
                    })[0];
                    cell.value = modifiedReportCell.value;
                    cell.isModified = true;
                    cell.modifiedReportCell = modifiedReportCell;
                    setData();
                });

                var cellValueResetFunc = $rootScope.$on('cellValueReset', function (event, args) {
                    var cell = scope.reportSource.filter(function (data) {
                        return data.rowIndex === args.row && data.columnIndex === args.col;
                    })[0];
                    cell.value = cell.modifiedReportCell?cell.modifiedReportCell.originalValue:cell.value;
                    cell.isModified = false;
                    cell.modifiedReportCell = null;
                    setData();
                });

                scope.$on('$destroy', function () {
                    layoutChangedFunc();
                    cellValueModifiedFunc();
                    cellValueResetFunc();
                });

                // Get spreadJS control.
                var getSpreadControl = function () {
                    return new GC.Spread.Sheets.Workbook(getSpreadControlElement());
                };

                // Get spreadJS control element
                var getSpreadControlElement = function () {
                    //var elem = document.getElementsByClassName('report-view');
                    var elem = document.getElementById('r_' + scope.templateId);
                    if (!elem) {
                        var elems = document.getElementsByClassName('report');
                        if (!elems || elems.length === 0) {
                            return undefined;
                        }
                        return elems[0];
                    }
                    return elem;
                };

                var repaintSpread = function () {
                    return $timeout(function () {
                        var spread = getSpreadControl();
                        if (spread && spread.refresh) {
                            spread.refresh();
                        }
                    }, 50);
                };

                // 根据已有信息通过spreadJS计算各单元格的值
                var setData = function () {
                    //todo: change to 0 at here,just for test now
                    var sheet = scope.spread.sheets[1];
                    var isExportData = false;
                    if (angular.isArray(scope.reportSource)) {
                        spreadJsTipService.initialize(sheet);
                        if (!scope.isReadOnly) {
                            setEditable(scope.spread, false, true);
                        }

                        scope.reportSource.forEach(function (data) {
                            //fix bug11737 导出需要显示千分位
                            // 避免直接使用data.value = parseFloat(data.value)导致非数字型value无法显示
                            data.value = PWC.tryParseStringToNum(data.value);

                            if (data.isExportData) {//导出报表时，设置单元格的公式信息
                                isExportData = true;
                                if (data.isFormula) {
                                    //获取单元格除公式值之外的剩余值
                                    var subValue = 0;
                                    if (data.dataSourceList && data.dataSourceList.length > 0) {
                                        data.dataSourceList.forEach(function (dataSource) {
                                            var cellName = PWC.numToExcelChar(dataSource.rowIndex, dataSource.columnIndex);
                                            if (dataSource.type != enums.formulaDataSourceType.Report && dataSource.amount) {
                                                subValue += parseFloat(dataSource.amount + '');
                                            }
                                        });
                                    }
                                    sheet.setFormula(data.rowIndex, data.columnIndex, '=' + data.formula + (subValue ? ('+' + subValue) : ''));
                                } else {
                                    sheet.setValue(data.rowIndex, data.columnIndex, data.value);
                                }

                            } else { // 显示报表时，设置单元格的值或公式
                                var cell = sheet.getCell(data.rowIndex, data.columnIndex);
                                if (data.isModified
                                    || (data.dataVoucherList && data.dataVoucherList.length)
                                    || (data.dataInvoiceList && data.dataInvoiceList.length)
                                    || (data.dataSourceList && data.dataSourceList.length
                                        && _.some(data.dataSourceList, function (dataSource) {
                                            return dataSource.dataSourceType === 6 && dataSource.amount
                                        }))
                                ) { // 存在用户手工输入值，需要改变底色
                                    cell.backColor('#fbe8cc');
                                }

                                if (!_.isEmpty(data.dataSourceList)) {
                                    var reports = _.chain(data.dataSourceList)
                                        .where({type: enums.formulaDataSourceType.Report})
                                        .map(function (x) {
                                            return {
                                                reportCode: x.reportCode,
                                                year: x.year,
                                                period: x.period
                                            };
                                        }).uniq(function (x) {
                                            return x.reportCode + ',' + x.year + ',' + x.period;
                                        }).value();
                                    if (reports.length === 1) {
                                        var drillDownArgs = _.findWhere(data.dataSourceList, {type: enums.formulaDataSourceType.Report});
                                        if (drillDownArgs.period === scope.projectPeriod && drillDownArgs.year === scope.projectYear) {
                                            cell.textDecoration(GcSpread.Sheets.TextDecorationType.Underline);
                                            cell.foreColor('blue');
                                        }
                                    }
                                }

                                // 设置顺序: 非数值字符串 -> 单元格值覆盖修改 -> 公式或手工数据源 -> 数值
                                var ifShowParseFloat = true; // 用于标识单元格是否设置成数值
                                if (_.isString(data.value) && data.value.length > 0 && isNaN(Number(data.value))) {
                                    sheet.setValue(data.rowIndex, data.columnIndex, data.value);
                                    ifShowParseFloat = false; // 非数值字符串，单元格设置为字符串
                                }
                                else if (data.isModified) { // 覆盖修改，且新值是数字
                                    if (_.isNumber(data.value) && !isNaN(data.value)) {
                                        data.value = data.value.toFixed(4);
                                    }

                                    sheet.setValue(data.rowIndex, data.columnIndex, data.value);
                                    ifShowParseFloat = false; // 单元格值覆盖修改
                                }
                                else if (!data.isReadOnly && !scope.isReadOnly) { // 非只读report(资料清单)且非只读单元格
                                    ifShowParseFloat = true;
                                }
                                else if (data.cellTemplateConfig) {
                                    var parsedFormula = data.cellTemplateConfig.parsedFormula;
                                    if (data.cellTemplateConfig.hasInvoice || data.cellTemplateConfig.hasKeyIn
                                        || data.cellTemplateConfig.hasModel || data.cellTemplateConfig.hasVoucher) {
                                        if (!_.isEmpty(parsedFormula)) {
                                            parsedFormula = parsedFormula + '+_Manual("' + data.cellTemplateID + '")';
                                        }
                                        else {
                                            parsedFormula = '_Manual("' + data.cellTemplateID + '")';
                                        }
                                    }

                                    if (!_.isEmpty(parsedFormula)) {
                                        parsedFormula = 'IFERROR(' + parsedFormula + ', "")';
                                        sheet.setFormula(data.rowIndex, data.columnIndex, '=' + parsedFormula);
                                        ifShowParseFloat = false; // 有公式或手工数据源，单元格设置为公式
                                    }
                                }

                                if (ifShowParseFloat) {
                                    if (_.isNumber(data.value) && !isNaN(data.value)) {
                                        data.value = data.value.toFixed(4);
                                    }

                                    sheet.setValue(data.rowIndex, data.columnIndex, data.value); // 只有数值，设置为数值
                                }
                            }

                            sheet.setTag(data.rowIndex, data.columnIndex, JSON.stringify(data));
                            //设置 tooltip 和 图标信息
                            spreadJsTipService.setCellTipByCellData(sheet.getCell(data.rowIndex, data.columnIndex), data);
                        });

                        // 设置破折号单元格的值和显示格式，将其替换为带破折号格式的0值，这样引用到的其他单元格才能正常计算
                        var rowCount = sheet.getRowCount();
                        var columnCount = sheet.getColumnCount();
                        for (var r = 0; r < rowCount; r++) {
                            for (var c = 0; c < columnCount; c++) {
                                var cell = sheet.getCell(r, c);
                                if (cell.value() === '——') {
                                    cell.value(0);
                                    cell.formatter("[=0]——;0.00");
                                }
                            }
                        }

                        spreadJsTipService.paintSheet(sheet);
                    }
                };

                // 更新数据源后，通过调用该方法计算所有当前sheet单元格，并通过比较原数据列表与spreadJS计算的新值列表，获取保存时需要修改库中Value的单元格信息列表
                var updateCells = function () {
                    setData();
                    var sheet = getSpreadControl().sheets[0];
                    var cells = [];
                    var reportDataSources = _.chain(scope.reportSource)
                        .pluck('dataSourceList')
                        .flatten(true).value();
                    angular.forEach(scope.reportSource, function (x) {
                        // 比较刷新前后报表中的值的变化
                        // 优先将单元格转换成数值比较，如果不能则转换为字符串比较
                        var cell = sheet.getCell(x.rowIndex, x.columnIndex);
                        var newVal = cell.value();
                        newVal = PWC.tryParseStringToNum(newVal);
                        if (_.isBoolean(newVal)) {
                            newVal = newVal.toString();
                        }
                        else if (_.isNumber(newVal) && !isNaN(newVal)) {
                            newVal = newVal.toFixed(4);
                        }

                        var oldVal = x.value;
                        oldVal = PWC.tryParseStringToNum(oldVal);
                        if (_.isBoolean(oldVal)) {
                            oldVal = oldVal.toString();
                        }
                        else if (_.isNumber(oldVal) && !isNaN(oldVal)) {
                            oldVal = oldVal.toFixed(4);
                        }

                        if (newVal !== oldVal) {
                            cells.push({id: x.cellID, oldVal: oldVal, newVal: newVal});
                            x.value = newVal;
                            x.isDirty = true;
                            var dirtyDataSources = _.filter(reportDataSources, {
                                cellDataID: x.cellID,
                                name: 'ReportDataSource'
                            });
                            angular.forEach(dirtyDataSources, function (ds) {
                                newVal = PWC.tryParseStringToNum(newVal);
                                ds.amount = newVal;
                            });

                            sheet.setTag(x.rowIndex, x.columnIndex, JSON.stringify(x));
                        }
                    });

                    return cells;
                };

                (function initialize() {
                    loadSheet(scope.templateId);
                    if (scope.internalApi) {
                        scope.internalApi.refreshReport = updateCells;
                    }
                })();

            }
        };
    }
]);